--
-- PostgreSQL database dump
--

\connect template1 postgres

DROP DATABASE hsgusertest;
CREATE DATABASE hsgusertest WITH OWNER = hsg TEMPLATE = template0 ENCODING = 'UNICODE';


\connect hsgusertest postgres

--

CREATE FUNCTION plpgsql_call_handler () RETURNS language_handler
    AS '$libdir/plpgsql', 'plpgsql_call_handler'
    LANGUAGE c;

CREATE TRUSTED PROCEDURAL LANGUAGE plpgsql HANDLER plpgsql_call_handler;


--

CREATE FUNCTION plperl_call_handler () RETURNS language_handler
    AS '$libdir/plperl', 'plperl_call_handler'
    LANGUAGE c;

CREATE PROCEDURAL LANGUAGE plperlu HANDLER plperl_call_handler;

REVOKE ALL ON LANGUAGE plperlu FROM PUBLIC;


--

CREATE FUNCTION enctype (text) RETURNS integer
    AS 'string2key', 'enctype'
    LANGUAGE c IMMUTABLE STRICT;

REVOKE ALL ON FUNCTION enctype (text) FROM PUBLIC;
GRANT ALL ON FUNCTION enctype (text) TO hsg;


--

CREATE FUNCTION salttype (text) RETURNS integer
    AS 'string2key', 'salttype'
    LANGUAGE c IMMUTABLE STRICT;


REVOKE ALL ON FUNCTION salttype (text) FROM PUBLIC;
GRANT ALL ON FUNCTION salttype (text) TO hsg;


--

CREATE FUNCTION string2key (text, text, integer, integer) RETURNS text
    AS 'string2key', 'string2key'
    LANGUAGE c IMMUTABLE STRICT;

REVOKE ALL ON FUNCTION string2key (text, text, integer, integer) FROM PUBLIC;
GRANT ALL ON FUNCTION string2key (text, text, integer, integer) TO hsg;


--

CREATE FUNCTION rsa_encrypt (text, text) RETURNS text
    AS '
use strict;
use Crypt::OpenSSL::RSA;
use MIME::Base64;

my ($data, $rsa_string) = @_;
my $ret;

my $rsa_pub =  Crypt::OpenSSL::RSA->new_public_key($rsa_string);
$ret = encode_base64($rsa_pub->encrypt($data), q[]);
$rsa_pub->DESTROY();

return $ret;
'
    LANGUAGE plperlu IMMUTABLE STRICT;

REVOKE ALL ON FUNCTION rsa_encrypt (text, text) FROM PUBLIC;
GRANT ALL ON FUNCTION rsa_encrypt (text, text) TO hsg;


--

CREATE FUNCTION lmpw_encrypt (text) RETURNS text
    AS '
use Crypt::SmbHash;
use Encode;
return Crypt::SmbHash::lmhash(decode_utf8($_[0]));
'
    LANGUAGE plperlu IMMUTABLE STRICT;

REVOKE ALL ON FUNCTION lmpw_encrypt (text) FROM PUBLIC;
GRANT ALL ON FUNCTION lmpw_encrypt (text) TO hsg;


--

CREATE FUNCTION ntpw_encrypt (text) RETURNS text
    AS '
use Crypt::SmbHash;
use Encode;
return Crypt::SmbHash::nthash(decode_utf8($_[0]));
'
    LANGUAGE plperlu IMMUTABLE STRICT;

REVOKE ALL ON FUNCTION ntpw_encrypt (text) FROM PUBLIC;
GRANT ALL ON FUNCTION ntpw_encrypt (text) TO hsg;


--

CREATE FUNCTION pwgen () RETURNS text
    AS '
use Crypt::PassGen qw/ passgen /;

my ($pw) = passgen(NLETT => 7);
my $uc_pos = int(rand(length $pw));
my $dig = int(rand(10));
my $dig_pos = int(rand(length $pw)) + 1;

substr($pw, $uc_pos, 1) = uc(substr($pw, $uc_pos, 1));
substr($pw, $dig_pos, 0) = $dig;

return $pw;
'
    LANGUAGE plperlu;


--

CREATE FUNCTION rsa_decrypt (text, text, text) RETURNS text
    AS '
use strict;
use Crypt::OpenSSL::RSA;
use MIME::Base64;

my ($data, $rsa_key_pw, $rsa_string) = @_;
my $ret;

my $rsa =  Crypt::OpenSSL::RSA->new_private_key($rsa_string, $rsa_key_pw);
$ret = $rsa->decrypt(decode_base64($data));
$rsa->DESTROY();

return $ret;
'
    LANGUAGE plperlu IMMUTABLE STRICT;

REVOKE ALL ON FUNCTION rsa_decrypt (text, text, text) FROM PUBLIC;
GRANT ALL ON FUNCTION rsa_decrypt (text, text, text) TO hsg;


--

CREATE FUNCTION sleep (integer) RETURNS void
    AS 'sleep($_[0]);'
    LANGUAGE plperlu;

REVOKE ALL ON FUNCTION sleep (integer) FROM PUBLIC;
GRANT ALL ON FUNCTION sleep (integer) TO hsg;


\connect - hsg

CREATE FUNCTION klasse_to_type (text) RETURNS text
    AS ' SELECT CASE
		WHEN $1 ~ ''^^([5-9]|10)[a-z]|1[1-3]$$'' THEN ''schueler''
		WHEN $1 = ''23e'' THEN ''lehrer''
		WHEN $1 = ''666a'' THEN ''admin''
		WHEN $1 = ''666b'' THEN ''machine''
		WHEN $1 = ''666c'' THEN ''nxx''
		WHEN $1 = ''666d'' THEN ''special''
	END AS RESULT;
	'
    LANGUAGE sql IMMUTABLE;

REVOKE ALL ON FUNCTION klasse_to_type (text) FROM PUBLIC;
GRANT ALL ON FUNCTION klasse_to_type (text) TO lpquota;
GRANT ALL ON FUNCTION klasse_to_type (text) TO nss;
GRANT ALL ON FUNCTION klasse_to_type (text) TO samba;
GRANT ALL ON FUNCTION klasse_to_type (text) TO sync_home;


--

CREATE FUNCTION min_uid () RETURNS integer
    AS 'SELECT 2000 AS RESULT;'
    LANGUAGE sql IMMUTABLE;

REVOKE ALL ON FUNCTION min_uid () FROM PUBLIC;


--

CREATE FUNCTION max_uid () RETURNS integer
    AS 'SELECT 4000 AS RESULT;'
    LANGUAGE sql IMMUTABLE;

REVOKE ALL ON FUNCTION max_uid () FROM PUBLIC;


--

CREATE FUNCTION uid_to_rid (integer) RETURNS integer
    AS 'SELECT CASE WHEN $1 = 0 THEN 500 ELSE $1*2 + 1000 END AS RESULT;'
    LANGUAGE sql IMMUTABLE;

REVOKE ALL ON FUNCTION uid_to_rid (integer) FROM PUBLIC;
GRANT ALL ON FUNCTION uid_to_rid (integer) TO samba;


--

CREATE FUNCTION gid_to_rid (integer) RETURNS integer
    AS 'SELECT CASE WHEN $1 = 0 THEN 512 ELSE $1*2 + 1001 END AS RESULT;'
    LANGUAGE sql IMMUTABLE;

REVOKE ALL ON FUNCTION gid_to_rid (integer) FROM PUBLIC;
GRANT ALL ON FUNCTION gid_to_rid (integer) TO samba;


--

CREATE FUNCTION rid_to_sid (integer) RETURNS text
    AS 'SELECT ''S-1-5-21-2976899603-3369439262-3073712670-'' || $1 AS RESULT;'
    LANGUAGE sql IMMUTABLE;


REVOKE ALL ON FUNCTION rid_to_sid (integer) FROM PUBLIC;
GRANT ALL ON FUNCTION rid_to_sid (integer) TO samba;


--

CREATE FUNCTION get_free_uid (integer) RETURNS integer
    AS '
	DECLARE
		idx ALIAS FOR $1;
		count INT;
		trash INT;
	BEGIN
		count := 0;
		FOR i IN min_uid()..max_uid() LOOP
			SELECT INTO trash 1 FROM users WHERE uid = i;
			IF NOT FOUND THEN
				IF count = idx THEN
					RETURN i;
				END IF;
			count := count + 1;
			END IF;
		END LOOP;
		RETURN NULL;
	END;'
    LANGUAGE plpgsql;

REVOKE ALL ON FUNCTION get_free_uid (integer) FROM PUBLIC;


--

CREATE FUNCTION login_normalize (text) RETURNS text
    AS '
	DECLARE
		name text;
		lparen int;
		rparen int;
	BEGIN
		name := lower($1);
		name := replace(name, convert(chr(228), ''LATIN1'', ''UNICODE''), ''ae'');
		name := replace(name, convert(chr(246), ''LATIN1'', ''UNICODE''), ''oe'');
		name := replace(name, convert(chr(252), ''LATIN1'', ''UNICODE''), ''ue'');
		name := replace(name, convert(chr(223), ''LATIN1'', ''UNICODE''), ''ss'');
		name := replace(name, convert(chr(232), ''LATIN1'', ''UNICODE''), ''e'');
		name := replace(name, convert(chr(233), ''LATIN1'', ''UNICODE''), ''e'');
		IF name LIKE ''de %'' THEN name := substring(name from 3); END IF;
		IF name LIKE ''von %'' THEN name := substring(name from 4); END IF;
		IF name LIKE ''dr. %'' THEN name := substring(name from 4); END IF;
		lparen := position(''('' in name);
		rparen := position('')'' in name);
		IF lparen != 0 AND rparen != 0 THEN
			name := substring(name from 1 for lparen-1) || substring(name from rparen+1);
		END IF;
		name := replace(name, '' '', '''');
		name := replace(name, ''-'', '''');
		RETURN name;
	END;'
    LANGUAGE plpgsql IMMUTABLE;

REVOKE ALL ON FUNCTION login_normalize (text) FROM PUBLIC;


--

CREATE FUNCTION login_merge (text, text, integer, integer) RETURNS text
    AS '
		SELECT substring($1 for $3) || substring($2 for $4);
	'
    LANGUAGE sql IMMUTABLE;

REVOKE ALL ON FUNCTION login_merge (text, text, integer, integer) FROM PUBLIC;


--

CREATE FUNCTION mk_login (text, text) RETURNS text
    AS '
	DECLARE
		vorname text;
		nachname text;
		testusername VARCHAR(8);
		trash INT;
	BEGIN
		---- normalize name
		vorname := login_normalize($1);
		nachname := login_normalize($2);
		---- search free username
		-- mix: 8 0
		testusername := login_merge(vorname, nachname, 8, 0);
		SELECT INTO trash 1 FROM users u WHERE u.username = testusername;
		IF NOT FOUND THEN RETURN testusername; END IF;
		-- mix: 7 0
		testusername := login_merge(vorname, nachname, 7, 0);
		SELECT INTO trash 1 FROM users u WHERE u.username = testusername;
		IF NOT FOUND THEN RETURN testusername; END IF;
		-- mix: 6 0
		testusername := login_merge(vorname, nachname, 6, 0);
		SELECT INTO trash 1 FROM users u WHERE u.username = testusername;
		IF NOT FOUND THEN RETURN testusername; END IF;
		-- mix: 7 1
		testusername := login_merge(vorname, nachname, 7, 1);
		SELECT INTO trash 1 FROM users u WHERE u.username = testusername;
		IF NOT FOUND THEN RETURN testusername; END IF;
		-- mix: 6 1
		testusername := login_merge(vorname, nachname, 6, 1);
		SELECT INTO trash 1 FROM users u WHERE u.username = testusername;
		IF NOT FOUND THEN RETURN testusername; END IF;
		-- mix: 6 2
		testusername := login_merge(vorname, nachname, 6, 2);
		SELECT INTO trash 1 FROM users u WHERE u.username = testusername;
		IF NOT FOUND THEN RETURN testusername; END IF;
		-- mix: 5 0
		testusername := login_merge(vorname, nachname, 5, 0);
		SELECT INTO trash 1 FROM users u WHERE u.username = testusername;
		IF NOT FOUND THEN RETURN testusername; END IF;
		-- mix: 3 5
		testusername := login_merge(vorname, nachname, 3, 5);
		SELECT INTO trash 1 FROM users u WHERE u.username = testusername;
		IF NOT FOUND THEN RETURN testusername; END IF;
		---- nothing found!
		RETURN NULL;
	END;'
    LANGUAGE plpgsql;

REVOKE ALL ON FUNCTION mk_login (text, text) FROM PUBLIC;


--

CREATE FUNCTION hsg_rsa_encrypt (text) RETURNS text
    AS '
	SELECT rsa_encrypt($1,''
-----BEGIN RSA PUBLIC KEY-----
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
-----END RSA PUBLIC KEY-----
	''); '
    LANGUAGE sql STRICT;

REVOKE ALL ON FUNCTION hsg_rsa_encrypt (text) FROM PUBLIC;


--

CREATE FUNCTION hsg_rsa_decrypt (text, text) RETURNS text
    AS '
	SELECT rsa_decrypt($1,$2,''
-----BEGIN RSA PRIVATE KEY-----
Proc-Type: 4,ENCRYPTED
DEK-Info: DES-EDE3-CBC,5578229B22D2AAF0

xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
-----END RSA PRIVATE KEY-----
	''); '
    LANGUAGE sql IMMUTABLE STRICT;


REVOKE ALL ON FUNCTION hsg_rsa_decrypt (text, text) FROM PUBLIC;


--

CREATE FUNCTION is_good_pw (text, integer, integer) RETURNS text
    AS '
	DECLARE
		pw ALIAS FOR $1;
		minlen ALIAS FOR $2;
		minclass ALIAS FOR $3;
		cnt INT;
	BEGIN
		IF length(pw) < minlen THEN
			RETURN ''tooshort'';
		END IF;
		IF pw ~ ''[^ -~]'' THEN
			RETURN ''invalidchar'';
		END IF;
		cnt := 0;
		IF pw ~ ''[a-z]'' THEN
			cnt := cnt + 1;
		END IF;
		IF pw ~ ''[A-Z]'' THEN
			cnt := cnt + 1;
		END IF;
		IF pw ~ ''[0-9]'' THEN
			cnt := cnt + 1;
		END IF;
		IF pw ~ ''[^0-9A-Za-z]'' THEN
			cnt := cnt + 1;
		END IF;
		IF cnt < minclass THEN
			RETURN ''toosimple'';
		END IF;
		RETURN ''ok'';
	END;'
    LANGUAGE plpgsql;

REVOKE ALL ON FUNCTION is_good_pw (text, integer, integer) FROM PUBLIC;


--

CREATE FUNCTION changepw (text, text, text) RETURNS text
    AS '
	DECLARE
		user ALIAS FOR $1;
		oldpw ALIAS FOR $2;
		newpw ALIAS FOR $3;
		status text;
	BEGIN
		PERFORM sleep(3);
		IF NOT testpw(user, oldpw) THEN
			RETURN ''wronglogin'';
		END IF;
		status := is_good_pw(newpw, 7, 3);
		IF NOT status = ''ok'' THEN
			RETURN status;
		END IF;
		PERFORM setpw(user, newpw);
		RETURN ''ok'';
	END;'
    LANGUAGE plpgsql SECURITY DEFINER;

REVOKE ALL ON FUNCTION changepw (text, text, text) FROM PUBLIC;
GRANT ALL ON FUNCTION changepw (text, text, text) TO chpasswd;


--

CREATE TABLE users (
    uid integer NOT NULL PRIMARY KEY,
    vorname text,
    nachname text NOT NULL,
    klasse character varying(4) NOT NULL,
    geburtstag date,
    username character varying(8) NOT NULL UNIQUE,
    lpused integer DEFAULT 0,
    lmpw text,
    ntpw text,
    "key" text,
    UNIQUE (vorname, nachname, geburtstag),
    CONSTRAINT users_klasse CHECK ((klasse_to_type((klasse)::text) IS NOT NULL)),
    CONSTRAINT users_lpused CHECK ((lpused >= 0)),
    CONSTRAINT users_uid CHECK (CASE WHEN (klasse_to_type((klasse)::text) <> 'admin'::text) THEN ((uid >= 2000) AND (uid <= 4000)) ELSE true END),
    CONSTRAINT users_username CHECK (CASE WHEN (username ~~ '%$'::text) THEN (username ~ '^[a-z][a-z0-9]{0,4}pc\\$$'::text) ELSE (username ~ '^[a-z]{1,8}$'::text) END),
    CONSTRAINT users_vorname CHECK (CASE WHEN (username ~~ '%$'::text) THEN (vorname = (username)::text) ELSE true END)
);

CREATE RULE users_update_synchome AS ON UPDATE TO users DO NOTIFY synchome_pg;
CREATE RULE users_insert_synchome AS ON INSERT TO users DO NOTIFY synchome_pg;
CREATE RULE users_delete_synchome AS ON DELETE TO users DO NOTIFY synchome_pg;


--

CREATE TABLE type_attrs (
    "type" text NOT NULL PRIMARY KEY,
    gid integer NOT NULL,
    dquota integer NOT NULL,
    lpquota integer NOT NULL,
    acct_flags text NOT NULL,
    home_path text,
    workstations text
);


--

CREATE VIEW nss AS
    SELECT username, 'x'::text AS passwd, uid, gid, ("type" || ' account'::text) AS gecos, ('/home/'::character varying || username) AS home, '/bin/bash'::text AS shell FROM (users JOIN type_attrs ON ((klasse_to_type((users.klasse)::text) = type_attrs."type"))) ORDER BY uid;

REVOKE ALL ON TABLE nss FROM PUBLIC;
GRANT SELECT ON TABLE nss TO nss;


--

CREATE VIEW samba AS
    SELECT username, ("type" || ' account'::text) AS gecos, rid_to_sid(uid_to_rid(uid)) AS sid, rid_to_sid(gid_to_rid(gid)) AS group_sid, acct_flags, 'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX'::text AS lmpw, ntpw, home_path, workstations FROM (users JOIN type_attrs ON ((klasse_to_type((users.klasse)::text) = type_attrs."type")));

REVOKE ALL ON TABLE samba FROM PUBLIC;
GRANT SELECT,UPDATE ON TABLE samba TO samba;

CREATE RULE samba_ntlmpw_update AS ON UPDATE TO samba DO INSTEAD UPDATE users SET lmpw = new.lmpw, ntpw = new.ntpw WHERE (users.username = old.username);


--

CREATE TABLE kerberos_keys (
    username character varying(8) NOT NULL REFERENCES users(username) ON UPDATE CASCADE ON DELETE CASCADE,
    enctype integer NOT NULL,
    salttype integer NOT NULL,
    "key" text NOT NULL,
    CONSTRAINT kerberos_keys_key CHECK (("key" ~ '^[0-9A-Za-z]+$'::text)),
    PRIMARY KEY (username, enctype, salttype)
);

REVOKE ALL ON TABLE kerberos_keys FROM PUBLIC;
GRANT SELECT ON TABLE kerberos_keys TO sync_kerb;

CREATE RULE kerberos_keys_insert_synckerb AS ON INSERT TO kerberos_keys DO NOTIFY kerberos_keys;
CREATE RULE kerberos_keys_update_synckerb AS ON UPDATE TO kerberos_keys DO NOTIFY kerberos_keys;
CREATE RULE kerberos_keys_delete_synckerb AS ON DELETE TO kerberos_keys DO NOTIFY kerberos_keys;


--

CREATE VIEW lpquota AS
    SELECT username, lpquota, lpused FROM (users JOIN type_attrs ON ((klasse_to_type((users.klasse)::text) = type_attrs."type")));

REVOKE ALL ON TABLE lpquota FROM PUBLIC;
GRANT SELECT,UPDATE ON TABLE lpquota TO lpquota;

CREATE RULE lpquote_lpused_update AS ON UPDATE TO lpquota DO INSTEAD UPDATE users SET lpused = new.lpused WHERE (users.username = old.username);


--

CREATE TABLE synchome_pg_special (
    username character varying(8) NOT NULL PRIMARY KEY REFERENCES users(username) ON UPDATE CASCADE ON DELETE CASCADE,
    b_soft integer,
    b_hard integer,
    i_soft integer,
    i_hard integer
);

CREATE RULE synchome_pg_special_update AS ON UPDATE TO synchome_pg_special DO NOTIFY synchome_pg;
CREATE RULE synchome_pg_special_delete AS ON DELETE TO synchome_pg_special DO NOTIFY synchome_pg;
CREATE RULE synchome_pg_special_insert AS ON INSERT TO synchome_pg_special DO NOTIFY synchome_pg;


--

CREATE VIEW synchome_pg AS
    SELECT uid, gid, username,
             COALESCE (b_soft, 0) AS b_soft, COALESCE (b_hard, dquota) AS b_hard,
             COALESCE (i_soft, 0) AS i_soft, COALESCE (i_hard, 0) AS i_hard
        FROM ((users JOIN type_attrs ON (klasse_to_type(users.klasse) = type_attrs."type"))
           LEFT JOIN synchome_pg_special USING (username))
        WHERE ((uid <> 0) AND ("type" <> 'machine'::text));

REVOKE ALL ON TABLE synchome_pg FROM PUBLIC;
GRANT SELECT ON TABLE synchome_pg TO sync_home;


--

CREATE FUNCTION mk_kerberos_key (text, text, text, text) RETURNS kerberos_keys
    AS '
    SELECT $1::varchar AS username, enctype($3) AS enctype, salttype($4) AS salttype, string2key($1, $2, enctype($3), salttype($4)) AS key;
'
    LANGUAGE sql STRICT;

REVOKE ALL ON FUNCTION mk_kerberos_key (text, text, text, text) FROM PUBLIC;


--

CREATE FUNCTION hsg_krb_encrypt (text, text) RETURNS SETOF kerberos_keys
    AS '
	      SELECT * FROM mk_kerberos_key($1, $2, ''des3-hmac-sha1'', ''normal''   )
	UNION SELECT * FROM mk_kerberos_key($1, $2, ''des-cbc-crc''   , ''normal''   )
	UNION SELECT * FROM mk_kerberos_key($1, $2, ''des''           , ''v4''       )
	UNION SELECT * FROM mk_kerberos_key($1, $2, ''des''           , ''norealm''  )
	UNION SELECT * FROM mk_kerberos_key($1, $2, ''des''           , ''onlyrealm'')
	UNION SELECT * FROM mk_kerberos_key($1, $2, ''des''           , ''afs3''     );
'
    LANGUAGE sql STRICT;

REVOKE ALL ON FUNCTION hsg_krb_encrypt (text, text) FROM PUBLIC;


--

CREATE FUNCTION setpw (text, text) RETURNS text
    AS '
	UPDATE users SET
		key = hsg_rsa_encrypt($2),
		lmpw = lmpw_encrypt($2),
		ntpw = ntpw_encrypt($2)
		WHERE username = $1;
	DELETE FROM kerberos_keys WHERE username = $1;
	INSERT INTO kerberos_keys SELECT * FROM hsg_krb_encrypt($1, $2);
	SELECT $2;
'
    LANGUAGE sql STRICT;

REVOKE ALL ON FUNCTION setpw (text, text) FROM PUBLIC;


--

CREATE FUNCTION testpw (text, text) RETURNS boolean
    AS 'SELECT COALESCE((SELECT ntpw = ntpw_encrypt($2) FROM users WHERE username = $1), FALSE);'
    LANGUAGE sql;

REVOKE ALL ON FUNCTION testpw (text, text) FROM PUBLIC;


--
